unit FreeOTFEfmeOptions_AutoRun;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, FreeOTFEfmeOptions_Base, FreeOTFESettings, StdCtrls, ComCtrls;

type
  TfmeOptions_Autorun = class(TfmeOptions_Base)
    gbAutorun: TGroupBox;
    Label33: TLabel;
    Label34: TLabel;
    Label35: TLabel;
    edPostMountExe: TEdit;
    pbPostMountBrowse: TButton;
    pbPreDismountBrowse: TButton;
    edPreDismountExe: TEdit;
    pbPostDismountBrowse: TButton;
    edPostDismountExe: TEdit;
    ckPrePostExeWarn: TCheckBox;
    OpenDialog: TOpenDialog;
    reInstructions: TRichEdit;
    procedure FrameResize(Sender: TObject);
    procedure pbPostMountBrowseClick(Sender: TObject);
    procedure pbPreDismountBrowseClick(Sender: TObject);
    procedure pbPostDismountBrowseClick(Sender: TObject);
  private
    { Private declarations }
  public
    procedure Initialize(); override;
    procedure ReadSettings(config: TFreeOTFESettings); override;
    procedure WriteSettings(config: TFreeOTFESettings); override;
  end;

implementation

{$R *.dfm}

uses
  SDUi18n,
  SDUGeneral;

resourcestring
  FILTER_EXECUTABLE_FILES = 'Program files (*.exe, *.com, *.bat)|*.exe; *.bat; *.com|All files|*.*';

procedure TfmeOptions_Autorun.FrameResize(Sender: TObject);
begin
  SDUCenterControl(gbAutorun, ccHorizontal);
  SDUCenterControl(gbAutorun, ccVertical, 25);
end;

procedure TfmeOptions_Autorun.pbPostMountBrowseClick(Sender: TObject);
begin
  inherited;
  OpenDialog.title := _('Locate post mount executable');
  OpenDialog.Filter := FILTER_EXECUTABLE_FILES;
  SDUOpenSaveDialogSetup(OpenDialog, edPostMountExe.text);
  if (OpenDialog.execute) then
    begin
    // Strip drive letter
    edPostMountExe.text := copy(OpenDialog.Filename, 3, length(OpenDialog.Filename)-2);
    end;

end;

procedure TfmeOptions_Autorun.pbPreDismountBrowseClick(Sender: TObject);
begin
  inherited;
  OpenDialog.title := _('Locate pre dismount executable');
  OpenDialog.Filter := FILTER_EXECUTABLE_FILES;
  SDUOpenSaveDialogSetup(OpenDialog, edPreDismountExe.text);
  if (OpenDialog.execute) then
    begin
    // Strip drive letter
    edPreDismountExe.text := copy(OpenDialog.Filename, 3, length(OpenDialog.Filename)-2);
    end;

end;

procedure TfmeOptions_Autorun.pbPostDismountBrowseClick(Sender: TObject);
begin
  inherited;
  OpenDialog.title := _('Locate post dismount executable');
  OpenDialog.Filter := FILTER_EXECUTABLE_FILES;
  SDUOpenSaveDialogSetup(OpenDialog, edPostDismountExe.text);
  if (OpenDialog.execute) then
    begin
    edPostDismountExe.text := OpenDialog.Filename;
    end;

end;

procedure TfmeOptions_Autorun.ReadSettings(config: TFreeOTFESettings);
begin
  edPostMountExe.text    := config.OptPostMountExe;
  edPreDismountExe.text  := config.OptPreDismountExe;
  edPostDismountExe.text := config.OptPostDismountExe;

  ckPrePostExeWarn.checked := config.OptPrePostExeWarn;

end;

procedure TfmeOptions_Autorun.WriteSettings(config: TFreeOTFESettings);
begin
  config.OptPostMountExe := edPostMountExe.text;
  config.OptPreDismountExe := edPreDismountExe.text;
  config.OptPostDismountExe := edPostDismountExe.text;

  config.OptPrePostExeWarn := ckPrePostExeWarn.checked;

end;

procedure TfmeOptions_Autorun.Initialize();
begin
  inherited;

  reInstructions.BorderStyle := bsNone;
  reInstructions.TabStop := FALSE;

  // Setup instructions text
  reInstructions.Lines.Clear();
  reInstructions.Lines.Add(_('FreeOTFE can be configured here to automatically run programs after mounting, before dismounting, and after dismounting.'));
  reInstructions.Lines.Add('');
  reInstructions.Lines.Add(_('Any post-mount and pre-dismount executables specified must use a relative path to the executable within the mounted volume (i.e. absolute and UNC paths are not allowed for security reasons)'));
  reInstructions.Lines.Add('');
  reInstructions.Lines.Add(_('Executables with spaces in their paths/names must be surrounded with double quotes.'));
  reInstructions.Lines.Add('');
  reInstructions.Lines.Add(_('"%DRIVE" will be substituted with the drive letter of the mounted drive letter'));

end;

END.

