unit FreeOTFEfmeOptions_General;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, FreeOTFEfmeOptions_Base, StdCtrls, ExtCtrls, Spin64,
  FreeOTFESettings;

type
  TLanguageTranslation = record
    Name: string;
    Code: string;
    Contact: string;
  end;
  PLanguageTranslation = ^TLanguageTranslation;

  TfmeOptions_General = class(TfmeOptions_Base)
    gbGeneral: TGroupBox;
    lblDragDrop: TLabel;
    ckAllowMultipleInstances: TCheckBox;
    ckDisplayToolbar: TCheckBox;
    ckDisplayStatusbar: TCheckBox;
    cbDragDrop: TComboBox;
    ckAutoStartPortable: TCheckBox;
    ckAdvancedMountDlg: TCheckBox;
    ckExploreAfterMount: TCheckBox;
    ckRevertVolTimestamps: TCheckBox;
    pnlVertSplit: TPanel;
    cbDrive: TComboBox;
    Label1: TLabel;
    seMRUMaxItemCount: TSpinEdit64;
    Label4: TLabel;
    lblMRUMaxItemCount: TLabel;
    ckShowPasswords: TCheckBox;
    cbLanguage: TComboBox;
    Label2: TLabel;
    pbLangDetails: TButton;
    procedure FrameResize(Sender: TObject);
    procedure pbLangDetailsClick(Sender: TObject);
    procedure cbLanguageChange(Sender: TObject);
  private
    FLanguages: array of TLanguageTranslation;

    procedure PopulateLanguages();
    procedure SetLanguageSelection(langCode: string);

    function  SelectedLanguage(): TLanguageTranslation;
    function  LanguageControlLanguage(idx: integer): TLanguageTranslation;
  public
    procedure Initialize(); override;
    procedure ReadSettings(config: TFreeOTFESettings); override;
    procedure WriteSettings(config: TFreeOTFESettings); override;
    procedure EnableDisableControls(); override;
  end;

implementation

{$R *.dfm}

uses
  SDUi18n,
  SDUGeneral,
  SDUDialogs,
  FreeOTFEfrmOptions;

const
  ISO639_ALPHA2_ENGLISH = 'en';
  ENGLISH_TRANSLATION_CONTACT = 'Sarah Dean <sdean12@sdean12.org>';

resourcestring
  USE_DEFAULT = 'Use default';

  RS_LANGUAGE = 'English'; // This should be set to the name of the language translated to

procedure TfmeOptions_General.cbLanguageChange(Sender: TObject);
var
  useLang: TLanguageTranslation;
  langIdx: integer;
begin
  inherited;

  // Preserve selected language; the selected language gets change by the
  // PopulateLanguages() call
  langIdx := cbLanguage.ItemIndex;

  useLang:= SelectedLanguage();
  TfrmOptions(Owner).ChangeLanguage(useLang.Code);
  // Repopulate the languages list; translation would have translated them all
  PopulateLanguages();

  // Restore selected
  cbLanguage.ItemIndex := langIdx;

  EnableDisableControls();
end;

procedure TfmeOptions_General.EnableDisableControls();
begin
  inherited;
  // Language at index 0 is "(Default)"
  SDUEnableControl(pbLangDetails, (cbLanguage.ItemIndex > 0));
end;

procedure TfmeOptions_General.FrameResize(Sender: TObject);
begin
  SDUCenterControl(gbGeneral, ccHorizontal);
  SDUCenterControl(gbGeneral, ccVertical, 25);
end;

procedure TfmeOptions_General.Initialize();
var
  driveLetter: char;
begin
  inherited;

  // Re-jig label size to take cater for differences in translation lengths
  // Size so the max. right is flush with the max right of pbLangDetails
  lblDragDrop.width        := (pbLangDetails.left + pbLangDetails.width) - lblDragDrop.left;
  lblMRUMaxItemCount.width := (pbLangDetails.left + pbLangDetails.width) - lblMRUMaxItemCount.left;

  pnlVertSplit.caption := '';
  pnlVertSplit.bevelouter := bvLowered;
  pnlVertSplit.width := 3;

  PopulateLanguages();

  cbDrive.Items.clear();
  cbDrive.Items.Add(USE_DEFAULT);
  for driveLetter:='C' to 'Z' do
    begin
    cbDrive.Items.Add(driveLetter+':');
    end;

end;

procedure TfmeOptions_General.pbLangDetailsClick(Sender: TObject);
var
  rcdLanguage: TLanguageTranslation;
begin
  inherited;

  rcdLanguage := SelectedLanguage();
  SDUMessageDlg(
                Format(_('Language name: %s'), [rcdLanguage.Name])+SDUCRLF+
                Format(_('Language code: %s'), [rcdLanguage.Code])+SDUCRLF+
                Format(_('Translator: %s'), [rcdLanguage.Contact])
               );

end;

procedure TfmeOptions_General.ReadSettings(config: TFreeOTFESettings);
var
  ft: TDragDropFileType;
  idx: integer;
  useIdx: integer;
begin
  // General...
  ckExploreAfterMount.checked := config.OptExploreAfterMount;
  ckAllowMultipleInstances.checked := config.OptAllowMultipleInstances;
  ckDisplayToolbar.checked := config.OptDisplayToolbar;
  ckDisplayStatusbar.checked := config.OptDisplayStatusbar;
  ckAutoStartPortable.checked := config.OptAutoStartPortable;
  ckAdvancedMountDlg.checked := config.OptAdvancedMountDlg;
  ckRevertVolTimestamps.checked := config.OptRevertVolTimestamps;
  ckShowPasswords.checked := config.OptShowPasswords;

  // In case language code not found; reset to "(Default)" entry; at index 0
  SetLanguageSelection(config.OptLanguageCode);

  // Populate and set drag drop filetype dropdown
  cbDragDrop.Items.Clear();
  idx := -1;
  useIdx := -1;
  for ft:=low(DragDropFileTypeTitle) to high(DragDropFileTypeTitle) do
    begin
    inc(idx);
    cbDragDrop.Items.Add(DragDropFileTypeTitle[ft]);
    if (config.OptDragDropFileType = ft) then
      begin
      useIdx := idx;
      end;
    end;
  cbDragDrop.ItemIndex := useIdx;

  // Default drive letter
  if (config.OptDefaultDriveLetter = #0) then
    begin
    cbDrive.ItemIndex := 0;
    end
  else
    begin
    cbDrive.ItemIndex := cbDrive.Items.IndexOf(config.OptDefaultDriveLetter+':');
    end;

  seMRUMaxItemCount.Value := config.OptMRUList.MaxItems;
end;


procedure TfmeOptions_General.WriteSettings(config: TFreeOTFESettings);
var
  ft: TDragDropFileType;
  useLang: TLanguageTranslation;
begin
  // General...
  config.OptExploreAfterMount := ckExploreAfterMount.checked;
  config.OptAllowMultipleInstances := ckAllowMultipleInstances.checked;
  config.OptDisplayToolbar := ckDisplayToolbar.checked;
  config.OptDisplayStatusbar := ckDisplayStatusbar.checked;
  config.OptAutoStartPortable := ckAutoStartPortable.checked;
  config.OptAdvancedMountDlg := ckAdvancedMountDlg.checked;
  config.OptRevertVolTimestamps := ckRevertVolTimestamps.checked;
  config.OptShowPasswords := ckShowPasswords.checked;

  useLang:= SelectedLanguage();
  config.OptLanguageCode := useLang.Code;

  // Decode drag drop filetype
  config.OptDragDropFileType := ftPrompt;
  for ft:=low(DragDropFileTypeTitle) to high(DragDropFileTypeTitle) do
    begin
    if (DragDropFileTypeTitle[ft] = cbDragDrop.Items[cbDragDrop.ItemIndex]) then
      begin
      config.OptDragDropFileType := ft;
      break;
      end;
    end;

  // Default drive letter
  if (cbDrive.ItemIndex = 0) then
    begin
    config.OptDefaultDriveLetter := #0;
    end
  else
    begin
    config.OptDefaultDriveLetter := cbDrive.Items[cbDrive.ItemIndex][1];
    end;

  config.OptMRUList.MaxItems := seMRUMaxItemCount.Value;

end;

procedure TfmeOptions_General.PopulateLanguages();
var
  i: integer;
  origLangCode: string;
  langCodes: TStringlist;
  sortedList: TStringList;
begin
  // Store language information for later use...
  langCodes:= TStringlist.Create();
  try
    // Get all language codes...
    SDUGetLanguageCodes(langCodes);

    // +1 to include "Default"
    SetLength(FLanguages, langCodes.count);

    // Spin though the languages, getting their corresponding human-readable
    // names
    origLangCode:= SDUGetCurrentLanguageCode();
    try
      for i:=0 to (langCodes.count - 1) do
        begin
        SDUSetLanguage(langCodes[i]);

        FLanguages[i].Code := langCodes[i];
        FLanguages[i].Name := _('English');
        FLanguages[i].Contact := SDUGetTranslatorNameAndEmail();

        // Force set contact details for English version; the dxgettext software sets
        // this to some stupid default
        if (langCodes[i] = ISO639_ALPHA2_ENGLISH) then
          begin
          FLanguages[i].Contact := ENGLISH_TRANSLATION_CONTACT;
          end;

        end;
    finally
      // Flip back to original language...
      SDUSetLanguage(origLangCode);
    end;

  finally
    langCodes.Free();
  end;

  // Add "default" into the list
  SetLength(FLanguages, (length(FLanguages) + 1));
  FLanguages[length(FLanguages)-1].Code := '';
  FLanguages[length(FLanguages)-1].Name := _('(Default)');
  FLanguages[length(FLanguages)-1].Contact := '';

  // Populate list
  sortedList:= TStringList.Create();
  try
    for i:=0 to (length(FLanguages) - 1) do
      begin
      sortedList.AddObject(FLanguages[i].Name, @(FLanguages[i]));
      end;

    sortedList.Sorted := TRUE;
    sortedList.Sorted := FALSE;

    cbLanguage.Items.Assign(sortedList)
  finally
    sortedList.Free();
  end;

end;

procedure TfmeOptions_General.SetLanguageSelection(langCode: string);
var
  useIdx: integer;
  currLang: TLanguageTranslation;
  i: integer;
begin
  useIdx := 0;
  for i:=0 to (cbLanguage.items.count - 1) do
    begin
    currLang := LanguageControlLanguage(i);
    if (currLang.Code = langCode) then
      begin
      useIdx := i;
      break
      end;
    end;
  cbLanguage.ItemIndex := useIdx;
end;

function TfmeOptions_General.SelectedLanguage(): TLanguageTranslation;
var
  retval: TLanguageTranslation;
begin
  retval := LanguageControlLanguage(cbLanguage.ItemIndex);
  Result := retval;
end;

function TfmeOptions_General.LanguageControlLanguage(idx: integer): TLanguageTranslation;
var
  retval: TLanguageTranslation;
begin
  retval := (PLanguageTranslation(cbLanguage.Items.Objects[idx]))^;
  Result := retval;
end;

END.

